/*******************************************************************************
                             Program ShowAllGlyphs
                                  20 May 2001
  It is assumed that both the type 42 version of the True Type font
  file and the file of glyph names, in ASCII sort order,
  are the current folder. this is done by running TrueTypeToType42.ps
  from the current directory. Batch file TTF2T42.bat provides a
  convenient means for doing this. Both of these two files have the
  PostScript name of the font as their root name. The type 42 file has
  extension .pfa and the list of glyphs has .glyphs as its extension.
  This program requires the PostScript name of the font as its sole
  command line argument. It produces a PostScript output file which,
  when copied to a PostScript printer, prints a list
  of all the glyphs and their names. The root name of this file is the
  PostScript name of the font and its extension is ps. You are given
  the option of including the type 42 version of the font file in its
  prologue. This adds considerably to the PostScript file size and
  need not be done if the font file is on the printer's hard disk.
  
*******************************************************************************/
#include <conio.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <windows.h>
#include <winbase.h>
#include <d:\utilities\thb.h>

void End(FILE *outfl, char outflnm[]);
void header(int nstr, int lstr, char *pstr);

void main(int argc, char *argv[])
{
  char GlyphName[50],hd[4][51],NGS[10],FontName[50];
  char fontflnm[120], inflnm[120], outflnm[120], strng[200];
  char *result;
  FILE *infl, *outfl, *fontfl;
  int  i, ipg, j, k;
  int NG, pgnum, xpg;

  strcpy(hd[0],"Welcome to ShowAllGlyphs, a utility for");
  strcpy(hd[1],"creating a PostScript program which");
  strcpy(hd[2],"will print all the glyphs of a font");
  strcpy(hd[3],"by T. H. Barton, May 2001");
  header(4,51,hd[0]);
/*
  Open the glyph names input file and the PostScript output file
*/
/*
  if(argc < 2) {
      Beep(880,500);
      printf(" Error message #1 from ShowAllGlyphs\n");
      printf(" argv[1] = %s\n",argv[1]);
      printf(" 1.  The file listing the glyphs must be in the current directory.\n");
      printf(" 2.  Its extension must be .glyphs.\n");
      printf(" 3.  You must supply its root name as the sole command line argument.\n");
    printf(" Please try again.\n");
    exit(0);
  }
*/
  result = strstr(argv[1],".glyphs");
  if(result == NULL) {
      Beep(880,500);
      printf(" Error message from ShowAllGlyphs\n");
      printf(" The command line argument is %s\n",argv[1]);
      printf(" It should end in .glyphs\n");
      printf(" Please correct and try again.\n");
      exit(0);
  }
  else {
      *result = 0;
  }
  strcpy(inflnm,argv[1]);
  strcat(inflnm,".glyphs");
  strcpy(outflnm,argv[1]);
  strcat(outflnm,".ps");
  strcpy(fontflnm,argv[1]);
  strcat(fontflnm,".pfa");
  printf(" Input file name  = %s\n",inflnm);
  printf(" Output file name = %s\n",outflnm);
  if((infl = fopen(inflnm,"r")) == NULL) {
    Beep(880,500);
    printf(" Error message #2 from ShowAllGlyphs\n");
    printf(" I could not open the input file %s\n",inflnm);
    printf(" It should be in the current folder.\n");
    printf(" Please check and try again.\n");
    exit(0);
  }
  if((outfl = fopen(outflnm,"w")) == NULL) {
    Beep(880,500);
    printf(" Error message #3 from ShowAllGlyphs\n");
    printf(" I could not open the output file %s for writing.\n",outflnm);
    printf(" Please check and try again.\n");
    exit(0);
  }
/*
  Read the PostScript name of the font and the number of glyphs
*/
  fgets(FontName,49,infl);
  j = strlen(FontName) - 1;
  FontName[j] = 0;
  fgets(NGS,9,infl);
  j = strlen(NGS) - 1;
  NGS[j] = 0;
/*
  Number of pages, 141 glyphs on first page, 150 on subsequent pages.
*/
  NG = atoi(NGS);
  printf(" Glyphs are printed in three columns per page.\n");
  printf(" Each column of the first page can hold 47 glyphs for a total of 141.\n");
  printf(" The columns of subsequent pages can contain 50 glyphs for a total of 150.\n\n");
  printf(" This font defines %d glyphs which are printed on ",NG);
  if(NG < 141) {
    ipg = 1;  // Partial first page
    xpg = 1;
    printf("one, partially full, page.\n");
  }
  else if(NG == 141) {
    ipg = 2; // Full first page
    xpg = 1;
    printf("one full page.\n");
  }
  else {
    i = (NG - 141)/150;
    j = (NG - 141) - i*150;
    if(j == 0)
    {
      ipg = 3;  // 1 + i full pages
      xpg = i + 1;
      printf("%d full pages.\n",xpg);
    }
    else {
      ipg = 4; // 1 + i full pages and one partial page
      xpg = i + 1;
      if (xpg == 1) {
        printf("%d full page and\n one partial page.\n",xpg);
      }
      else {
        printf("%d full pages and\n one partial page.\n",xpg);
      }
      xpg++;
    }
  }
/*
  Write the preliminaries to the PostScript output file
*/
  fprintf(outfl,"%%!PS-Adobe-3.0\n");
  fprintf(outfl,"%%%%Title:TNRGlyphs.ps\n");
  fprintf(outfl,"%%%%Creator: T.H.Barton\n");
  fprintf(outfl,"%%%%CreationDate: May 2001\n");
  fprintf(outfl,"%%%%BoundingBox: 0 0 612 792\n");
  fprintf(outfl,"%%%%Pages: %d\n",xpg);
  xpg--;
  fprintf(outfl,"%%%%EndComments\n");
  fprintf(outfl,"%%%%BeginProlog\n");
  fprintf(outfl,"/ctr{\n");
  fprintf(outfl,"  4 -2 roll 2 copy translate\n");
  fprintf(outfl,"  4 -2 roll exch dup rotate exch\n");
  fprintf(outfl,"  dup stringwidth pop 2 div neg 0 moveto show\n");
  fprintf(outfl,"  neg rotate\n");
  fprintf(outfl,"  neg exch neg exch translate\n");
  fprintf(outfl,"} def\n");
  fprintf(outfl,"/dl{moveto lineto stroke} def\n");
  fprintf(outfl,"/mt{moveto} def\n");
  fprintf(outfl,"/nl {\n");
  fprintf(outfl,"  count 0 eq {1.0}{\n");
  fprintf(outfl,"    dup type 20 string cvs dup\n");
  fprintf(outfl,"    (integertype) eq exch\n");
  fprintf(outfl,"    (realtype) eq xor not {1.0} if\n");
  fprintf(outfl,"  } ifelse\n");
  fprintf(outfl,"  dy mul neg yp add /yp exch store xp yp moveto\n");
  fprintf(outfl,"} def\n");
  fprintf(outfl,"/rj{\n");
  fprintf(outfl,"  4 -2 roll 2 copy translate\n");
  fprintf(outfl,"  4 -2 roll exch dup rotate exch\n");
  fprintf(outfl,"  dup stringwidth pop neg 0 moveto show\n");
  fprintf(outfl,"  neg rotate\n");
  fprintf(outfl,"  neg exch neg exch translate\n");
  fprintf(outfl,"} def\n");
  fprintf(outfl,"/sg{setgray} def\n");
  fprintf(outfl,"/PB1 {\n");
  fprintf(outfl,"  0.7 sg 2.0 slw 36 36 540 720 rectstroke\n");
  fprintf(outfl,"  36 711 576 711 dl\n");
  fprintf(outfl,"  216 36 216 711 dl 396 36 396 711 dl 0 sg\n");
  fprintf(outfl,"} def\n");
  fprintf(outfl,"/PBX {\n");
  fprintf(outfl,"  0.7 sg 2.0 slw 36 36 540 720 rectstroke\n");
  fprintf(outfl,"  216 36 216 756 dl 396 36 396 756 dl 0 sg\n");
  fprintf(outfl,"} def\n");
  fprintf(outfl,"/sh{show} def\n");
  fprintf(outfl,"/slw{setlinewidth} def\n");
  fprintf(outfl,"/F1 {/Times-Roman 12 selectfont} def\n");
  fprintf(outfl,"/F2 {/%s 12 selectfont} def\n",FontName);

  printf("\n Do you want to include the Type 42 font file in the output file?   y/n   ");
  i = getche();
  if(i == 89 || i == 121) {
    fprintf(outfl,"%% Start of the font PFA file\n");
    if((fontfl = fopen(fontflnm,"r")) == NULL) {
      Beep(880,500);
      printf(" Error message #4 from ShowAllGlyphs\n");
      printf(" I could not open the font file %s\n",fontflnm);
      printf(" It should be in the current folder.\n");
      printf(" Please check and try again.\n");
      exit(0);
    }
    while(fgets(strng,200,fontfl) != NULL) {
      fprintf(outfl,"%s",strng);
    }
    fprintf(outfl,"%% End of font PFA file\n");
    fclose(fontfl);
  }

  fprintf(outfl,"%%%%EndProlog\n");
  fprintf(outfl,"%%%%BeginSetup\n");
  fprintf(outfl,"<<\n");
  fprintf(outfl,"%%   /Collate true\n");
  printf("\n Do you want duplex or simplex printing? (Defaults to simplex)   d/s   \n");
  i = getche();
  printf("\n");
  if(i == 68 || i == 100) {
    fprintf(outfl,"  /Duplex true\n");
    fprintf(outfl,"  /MediaPosition 2  %% 2 = bottom tray, 3 = manual feed tray\n");
  }
  else {
    fprintf(outfl,"%%   /Duplex true\n");
    fprintf(outfl,"  /MediaPosition 3  %% 2 = bottom tray, 3 = manual feed tray\n");
  }
  fprintf(outfl,"%%   /HWResolution [1200 1200]\n");
  fprintf(outfl,"%%   /ManualFeed true\n");
  fprintf(outfl,"%%   /PageSize [792 612]\n");
  fprintf(outfl,">> setpagedevice\n");
  fprintf(outfl,"%%\n");
  fprintf(outfl,"%%%%EndSetup\n");
  fprintf(outfl,"/#copies 1 def\n");
/*
  First page
*/
  pgnum = 1;
  fprintf(outfl,"%%%%Page: %d %d\n",pgnum,pgnum);
  fprintf(outfl,"/ypo 710 def /dy 14 def\n");
  fprintf(outfl,"/xp 42 def /x1 xp 147 add def\n");
  fprintf(outfl,"/x2 xp 180 add def /x3 x2 147 add def\n");
  fprintf(outfl,"/x4 xp 360 add def /x5 x4 147 add def\n");
  fprintf(outfl,"PB1\n");
/*
  Document header
*/
  fprintf(outfl,"/Times-Bold 18 selectfont\n");
  fprintf(outfl," 306 738 0 (%s Glyphs) ctr\n",FontName);
  fprintf(outfl,"/Times-Bold 12 selectfont\n");
  fprintf(outfl,"306 723 0 (All %s glyphs are shown in ASCII sort order) ctr\n\n",NGS);
  fprintf(outfl,"/%s 12 selectfont\n",FontName);
/*
  First page has three columns with 47 glyphs per column for a total of
  141 glyphs.
*/
  fprintf(outfl,"/yp ypo def\n");
  for(i = 0; i <= 47; i++) {
    if(fgets(GlyphName,49,infl) == NULL) End(outfl, outflnm);
    j = strlen(GlyphName) - 1;
    GlyphName[j] = 0;
    fprintf(outfl,"1.0 nl F1 (%s) sh x1 yp mt F2 /%s glyphshow\n",GlyphName,GlyphName);
  }
  fprintf(outfl,"/yp ypo def\n");
  for(i = 0; i <= 47; i++) {
    if(fgets(GlyphName,49,infl) == NULL) End(outfl, outflnm);
    j = strlen(GlyphName) - 1;
    GlyphName[j] = 0;
    fprintf(outfl,"1.0 nl x2 yp mt F1 (%s) sh x3 yp mt F2 /%s glyphshow\n",GlyphName,GlyphName);
  }
  fprintf(outfl,"/yp ypo def\n");
  for(i = 0; i <= 47; i++) {
    if(fgets(GlyphName,49,infl) == NULL) End(outfl, outflnm);
    j = strlen(GlyphName) - 1;
    GlyphName[j] = 0;
    fprintf(outfl,"1.0 nl x4 yp mt F1 (%s) sh x5 yp mt F2 /%s glyphshow\n",GlyphName,GlyphName);
  }
  fprintf(outfl,"showpage\n");
/*
  Subsequent pages have three columns with 50 glyphs per column for a
  total of 150 glyphs per page.
*/
  for(k = 1; k <= xpg; k++) {
    pgnum++;
    fprintf(outfl,"%%%%Page: %d %d\n",pgnum,pgnum);
    fprintf(outfl,"/ypo 756 def /dy 14 def\n");
    fprintf(outfl,"/xp 42 def /x1 xp 147 add def\n");
    fprintf(outfl,"/x2 xp 180 add def /x3 x2 147 add def\n");
    fprintf(outfl,"/x4 xp 360 add def /x5 x4 147 add def\n");
    fprintf(outfl,"PBX\n");
    fprintf(outfl,"/Helvetica 12 selectfont\n");
    fprintf(outfl,"36 764 mt (Glyphs of %s) sh\n",FontName);
    fprintf(outfl,"576 764 0 (Page %d) rj\n",pgnum);
    fprintf(outfl,"/%s 12 selectfont\n",FontName);
    fprintf(outfl,"/yp ypo def\n");
    for(i = 0; i <= 50; i++) {
      if(fgets(GlyphName,49,infl) == NULL) End(outfl, outflnm);
      j = strlen(GlyphName) - 1;
      GlyphName[j] = 0;
      fprintf(outfl,"1.0 nl F1 (%s) sh x1 yp mt F2 /%s glyphshow\n",GlyphName,GlyphName);
    }
    fprintf(outfl,"/yp ypo def\n");
    for(i = 0; i <= 50; i++) {
      if(fgets(GlyphName,49,infl) == NULL) End(outfl, outflnm);
      j = strlen(GlyphName) - 1;
      GlyphName[j] = 0;
      fprintf(outfl,"1.0 nl x2 yp mt F1 (%s) sh x3 yp mt F2 /%s glyphshow\n",GlyphName,GlyphName);
    }
    fprintf(outfl,"/yp ypo def\n");
    for(i = 0; i <= 50; i++) {
      if(fgets(GlyphName,49,infl) == NULL) End(outfl, outflnm);
      j = strlen(GlyphName) - 1;
      GlyphName[j] = 0;
      fprintf(outfl,"1.0 nl x4 yp mt F1 (%s) sh x5 yp mt F2 /%s glyphshow\n",GlyphName,GlyphName);
    }
    fprintf(outfl,"showpage\n");
  }
  exit(0);
}
void End(FILE *outfl, char outflnm[])
{
  fprintf(outfl,"showpage\n");
  fprintf(outfl,"%%%%Trailer\n");
  fprintf(outfl,"%%%%EOF");
  printf("\n I have completed construction of output file:\n   %s\n",outflnm);
  printf(" Press any key to continue.\n");
  getch();
  exit(0);
}
/*****************************************************************************

                                    HEADER.C
                            SUN, 16 SEP 1990, 6:20
  Program for displaying a header in gray with nstr lines of text.  Ptr is
  a pointer to the text.  Header first clears the screen and then starts on
  line 1, centers the header on the screen and surrounds the header by a
  double border line.  Header leaves the cursor at the start of the line
  two lines below the header.

  Inputs
     nstr    number of strings
     lstr    length of string as declared in calling program
     pstr    pointer to first character of header string array

  The declaration is ... void header(int, int, char *ptr);

*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

void header(int nstr, int lstr, char *pstr)
{
    char   *ptr;
    int    i, imax = 0, j, nch[9], nchmax = 0;
    int    xst, yst;

    system("cls");
    for(i = 0; i < nstr; i++)                /* Get lengths of header strings */
    {
        ptr = pstr + lstr*i;
        nch[i] = strlen(ptr);
        if(nch[i] > nchmax)
        {
            imax = i;
            nchmax = nch[i];
        }
    }
    nchmax = nchmax + 6;           /* Leave three blanks round longest string */
    xst = (80 - nchmax)/2;
    yst = 1;

    for(i = 0; i < xst; i++) printf(" ");
    printf("%c",201);
    for(i = 1; i <= nchmax; i++) printf("%c",205);
    printf("%c\n",187);

    for(j = 0; j < nstr; j++)                              /* Display strings */
    {
        for(i = 0; i < xst; i++) printf(" ");
        ptr = pstr + lstr*j;
        yst++;
        printf("%c",186);
        imax = (nchmax - nch[j])/2;
        for(i = 1; i <= imax; i++) printf(" ");
        printf("%s",ptr);
        imax = imax + nch[j];
        for(i = imax; i < nchmax; i++) printf(" ");
        printf("%c\n",186);
    }

    for(i = 0; i < xst; i++) printf(" ");
    printf("%c",200);
    for(i = 1; i <= nchmax; i++) printf("%c",205);
    printf("%c\n\n",188);
}
