echo off
REM TTF2T42.bat is a Windows batch file intended to run GhostScript on
REM file TrueTypeToType42.ps in order to convert the TrueType font file
REM specified by the single command line argument to a PostScript type 42
REM font file. It generates the Type 42 file and also a list of its
REM glyphs in ASCII sort order. The name of the font file is the PostScript
REM name of the font with extension PFA. The root name of the glyph list
REM file is the PostScript name of the font with the extension .glyphs.
REM For example, if the TrueType file is Times.ttf and its PostScript name
REM is TimesNewRomanPSMT, its Type 42 file is called TimesNewRomanPSMT.pfa
REM and its glyph file is TimesNewRomanPSMT.glyphs.
REM The PFA file can be used like a Type 1 PFA file:
REM    it can be inserted as-is into the prologue of any PostScript file using the font.
REM    it can be downloaded as-is to a printer hard disk.
REM    it can be renamed REM and/or reencoded.
REM In the glyph file:
REM    the first line gives the PostScript name of the font,
REM    the second line gives the number of glyphs.
REM    the remaining lines contain the glyph names in ASCII sort order,
REM    one glyph name per line.
REM I keep this batch file in directory C:\BatchFiles which is on my
REM path so that it can be run by issuing its name, TTF2T42, at the
REM command prompt of the current directory. It requires one command
REM line argument. In general, this is the full path name of the TTF file
REM minus its extension, .TTF. If the current folder contains the TTF file,
REM its root name will suffice. It is assumed that the path to the
REM GhostScript executable is:
REM    C:\GS\GS7.00\bin\gswin32
REM and that the TrueType fonts are stored in:
REM    C:\Winnt\Fonts
REM These two items should be changed to suit individual installationS.
REM Depending on the number of glyphs defined by the font, the program may
REM take several minutes to run and you are given lots of informative reports
REM on progress in the GhostScript stdout window.
REM Most of the running time is taken up by sorting the glyph names
REM into ASCII order.
REM The program creates two files in the current directory, one
REM whose root name is the PostScript name of the font with extension
REM PFA and other with the same root name and extension glyphs.
REM Since type 42 fonts were only introduced about half way through the
REM life of PostScript 2, you must use a recent version of GhostScript.
REM I have successfully used it with GhostScript 6.50 and 7.00
if "%1"=="" goto G1
if not exist C:\Winnt\Fonts\%1.ttf goto G2
C:\GS\GS7.00\bin\gswin32 -IC:\GS\GS7.00 -- D:\Fonts\TrueType\CtanFiles\TrueTypeToType42.ps C:\TrueTypeFonts\%1.ttf
goto End
:G1 echo  You must give the root name of the TTF file as the command
echo line argument. Please try again.
goto End
:G2 echo  I cannot file the TrueType input file %1.ttf in folder
echo C:\Winnt\Fonts. Please check and try again.
goto End
:End